/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Reseau.h"
#include <unistd.h>

void Get_My_MAC (int fd, char *Interface)
{
	struct ifreq ifr;
	char      AdresseMac[18];

	ifr.ifr_addr.sa_family = AF_INET;
	sprintf (ifr.ifr_name, Interface);
	if (ioctl (fd, SIOCGIFHWADDR, &ifr) == -1) {
		perror ("[S] Erreur : Recuperation  @Mac");
		Message ("Error : Mac: No such device", ERREUR);
		Message ("", AUTOSCAN_SEGMENTATION_FAULT);
		exit (1);
	}
	memcpy (MonAdresseMac, ifr.ifr_hwaddr.sa_data, ETH_ALEN);
	sprintf (AdresseMac, "%02X:%02X:%02X:%02X:%02X:%02X", MonAdresseMac[0], MonAdresseMac[1], MonAdresseMac[2], MonAdresseMac[3], MonAdresseMac[4], MonAdresseMac[5]);
	printf ("[S]  *  Adresse Mac : %s\n", AdresseMac);
}

void Get_My_Ip (int fd, char *Interface)
{
	struct ifreq ifr;
	struct sockaddr_in sin;
	struct in_addr in;
	unsigned long int *pAdresseIp;

	strcpy (ifr.ifr_name, Interface);
	if (ioctl (fd, SIOCGIFADDR, &ifr) == -1) {
		perror ("[S] Erreur : Recuperation @Ip");
		Message ("Error : Cannot assign requested address", ERREUR);
		Message ("", AUTOSCAN_SEGMENTATION_FAULT);
		exit(1);
	}
	memcpy (&sin, &ifr.ifr_addr, sizeof (struct sockaddr_in));
	MonAdresseIp = (unsigned long int) sin.sin_addr.s_addr;
	in.s_addr = sin.sin_addr.s_addr;
	strcpy (Mon_Adresse_Ip, inet_ntoa (in));
	printf ("[S]  *  Interface IP: %s\n", Mon_Adresse_Ip);
}

void Get_Sous_Reseau (int fd, char *Interface)
{
	struct ifreq ifr;
	struct sockaddr_in sin;
	unsigned long int in;
	unsigned char Masque[4];
	int       i;

	strcpy (ifr.ifr_name, Interface);
	if (ioctl (fd, SIOCGIFBRDADDR, &ifr) == -1) {
		perror ("[S] Erreur : Broacast");
		exit (1);
	}
	memcpy (&sin, &ifr.ifr_broadaddr, sizeof (struct sockaddr_in));
	in = sin.sin_addr.s_addr;
	memcpy (&Masque, &in, 4);
	printf ("[S] Masque  : ");
	i = 0;
	while ((i < 3) && (Masque[i] != 255)) {
		printf ("%d.", Masque[i]);
		i++;
	}
	printf ("\n");
}

void Liste_Interface ()
{
	FILE     *Fichier_Interface;
	char      buf[512], *debut, *fin;

	if ((Fichier_Interface = fopen ("/proc/net/dev", "r")) == NULL) {
		perror ("[S] fopen :");
		printf ("[S] Impossible de lire /proc/net/dev\n");
		exit (-1);
	}
	fgets (buf, sizeof (buf), Fichier_Interface);
	fgets (buf, sizeof (buf), Fichier_Interface);
	while (fgets (buf, sizeof (buf), Fichier_Interface)) {
		debut = buf;
		while (debut[0] == ' ')
			debut++;
		fin = index (buf, ':');
		fin[0] = 0;
		printf ("%s\n", debut);
	}
}

/*
* Creation d'une socket
*  ------------------------------------
*  retourne le num de la socket
*  Gestion de erreurs : FALSE
*/
int Creation_Socket_TCP ()
{
	int       sock;

	sock = socket (AF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		perror ("[S] Erreur Creation socket TCP/IP");
		return FALSE;
	}
	printf ("[S] Creation d'une socket TCP/IP\n");
	return sock;
}

/*
*  connecte au serveur
* ------------------------------------
*  serveur : nom du serveur , host
*  port : num du port , sock : num de la socket
*  Gestion de erreurs : FALSE
*/
int connection_Serveur (char *serveur, struct hostent *host, int port, int sock)// boolean
{
	struct sockaddr_in SocketAddr;
	memset (&SocketAddr, 0, sizeof (struct sockaddr_in));
	memcpy (&SocketAddr.sin_addr.s_addr, host->h_addr, host->h_length);
	SocketAddr.sin_family = AF_INET;
	SocketAddr.sin_port = htons (port);
	printf ("[S] Connection au serveur [%s] en cours\n", serveur);
	if (connect (sock, (struct sockaddr *) &SocketAddr, sizeof (struct sockaddr)) < 0) {
		if (errno != EINPROGRESS) {
			perror ("[S] Erreur de Connection");
			return FALSE;// boolean
		}
		if (waiting_sock (sock, 1000) == FALSE)
			return FALSE;// boolean
	}
	printf ("[S] Connection au serveur : %s:%i\n", serveur, port);
	return TRUE;// boolean
}





/*
*  Resout le DNS
*  ------------------------
*  serveur : nom du serveur
*  retourne une structure de type hostent
*  Gestion de erreurs : retourne FALSE
*/
struct hostent *resoudre_DNS (char *serveur)
{
	struct hostent *host;

	host = gethostbyname (serveur);	// recupere l'adresse IP
	if (host == NULL) {
		switch (h_errno) {
		    case HOST_NOT_FOUND:
			    printf ("[S] L'hote indique est inconnu.\n");
			    break;
		    case NO_ADDRESS:
			    printf ("[S] Le nom est valide mais ne possède pas d'adresse IP.\n");
			    break;
		    case NO_RECOVERY:
			    printf ("[S] Une erreur fatale du serveur de noms est apparue.\n");
			    break;
		    case TRY_AGAIN:
			    printf ("[S] Aucune connection a internet.\n");
			    break;
		    default:
			    printf ("[S] Erreur Inconnue\n");
			    break;
		}
		return FALSE;
	}
	return host;
}


/*
*  Modifie le descripteur pour qu'il soit non bloquant
* ------------------------------------
*/

void set_NonBlock (int sock)
{
	int       flags;

	if ((flags = fcntl (sock, F_GETFL)) == -1) {
		perror ("[S] Set_NonBlock get");
		exit (-1);
	}
	if (fcntl (sock, F_SETFL, flags | O_NONBLOCK) == -1) {
		perror ("[S] Set_NonBlock set");
		exit (-1);
	}
}

/*
*  attend un evenement sur un descripteur
* ------------------------------------
*  sock : num de la socket
*  Gestion de erreurs : FALSE apres le timeout de 300 secondes
*/
gboolean waiting_donnee (int sock, int pause)
{
	struct pollfd fds;
	int       resultat;

	bzero (&fds, sizeof fds);	// on mets la structure a zero
	fds.events = POLLIN;
	fds.fd = sock;		// je scrute mon socket
	if ((resultat = poll (&fds, 1, pause)) > 0) {	// attend 1 seconde
		return TRUE;
	}
	else {
		if (resultat != 0) {
			perror ("[S] Erreur de poll");
		}
		return FALSE;
	}
}




/*
*  attend un evenement sur un descripteur
* ------------------------------------
*  sock : num de la socket
*  Gestion de erreurs : FALSE apres le timeout de 300 secondes
*/
gboolean waiting_sock (int sock, int pause)// boolean
{
	struct pollfd fds;
	int       resultat;

	bzero (&fds, sizeof fds);	// on mets la structure a zero
	fds.events = POLLOUT;
	fds.fd = sock;		// je scrute mon socket
	if ((resultat = poll (&fds, 1, pause)) > 0) {	// attend 1 seconde
		return TRUE;// boolean
	}
	else {
		if (resultat != 0) {
			perror ("[S] Erreur de poll");
		}
		return FALSE;// boolean
	}
}

/*
*  Ouverture du flux associ  une socket
* ------------------------------------
*  sock : num de la socket, le descripteur de fichier associ  la socket
*  Gestion de erreurs : FALSE
*/
FILE     *Ouverture_Flux_Socket (int sock)
{
	FILE     *In;

	In = fdopen (sock, "r");
	if (In == NULL) {
		perror ("[S] Erreur  l'ouverture du flux en lecture");
		return FALSE;// boolean ???
	}
	return In;
}

unsigned short ckecksum (char *donnee, int longueur,int valeur)
{
	int       i;
	int       somme;
	int       Nombre;

	i = 0;
	somme = 0;
	while (i < longueur) {
		Nombre = (((donnee[i] << 8) & 0xFF00) + (donnee[i + 1] & 0xFF));
		somme = somme + Nombre;
		i = i + 2;
	}
	while ((somme >> 16) != 0) {
		somme = (somme >> 16) + (somme & 0xffff);
	}
	somme = (0xFFFF - (somme & 0xFFFF)) - valeur;
	if ((valeur!=0)&&(somme < 0))
		somme--;
	return (unsigned short)somme;
}
