/*
 *  Copyright (C) 2004 Thierry Lagarde <tlagarde@user.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "Reseau.h"
#include "callbacks.h"
#include "Commun.h"
#include "Machine.h"
#include "Smb.h"
#include <time.h>


void Initialise_Structure_Domaine ()
{
	int       i;

	Nb_Domaine_Serveur = 0;
	Nb_Domaine_Station = 0;
	Nb_Station = 0;
	Nb_Serveur = 0;
	for (i = 0; i < MAX_DOMAINE; i++)
		Liste_Domaines_Serveur[i].ctree = NULL;
	for (i = 0; i < MAX_DOMAINE; i++)
		Liste_Domaines_Station[i].ctree = NULL;
}


void Initialise_Structure_SMB (int NumMachine)
{
	int       i;

	Liste_Machines[NumMachine].Smb.HostName[0] = 0;
	Liste_Machines[NumMachine].Smb.DomainName[0] = 0;
	Liste_Machines[NumMachine].Smb.Utilisateur[0] = 0;
	for (i = 0; i < NUM_MAX_REGISTRATION; i++)
		Liste_Machines[NumMachine].Smb.Smb_Registration[i].Type = 0;
}

void Recuperation_Information_NetBios (int Machine, int Num_Enregistrement)
{
	/* recupere le nom de la machine */
	if ((Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Numero == 0x20) && (Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Type = REGISTRATION_UNIQUE))
		strcpy (Liste_Machines[Machine].Smb.HostName, Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Name);
	/* recupere le nom du domaine */
	if ((Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Numero == 0x00) && (Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Type == REGISTRATION_GROUP))
		strcpy (Liste_Machines[Machine].Smb.DomainName, Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Name);
	/* recupere le nom de l'utilisateur */
	if ((Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Numero == 0x03) && (Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Type == REGISTRATION_UNIQUE))
		strcpy (Liste_Machines[Machine].Smb.Utilisateur, Liste_Machines[Machine].Smb.Smb_Registration[Num_Enregistrement].Name);

}


void Creer_Node_Serveur ()
{
	gchar    *text[1];

	if (Node_Serveur == NULL) {
		text[0] = g_strdup_printf ("Server");
		Node_Serveur = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), NULL, NULL, text, 5, Pixmap_Serveur, Masque_Serveur, Pixmap_Serveur, Masque_Serveur, FALSE, FALSE);
		gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Node_Serveur, (gpointer *) SERVEUR);
		g_free (text[0]);
	}
}


void Creer_Node_Station ()
{
	gchar    *text[1];

	if (Node_Station == NULL) {
		text[0] = g_strdup_printf ("Workstation");
		Node_Station = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), NULL, NULL, text, 5, Pixmap_Station, Masque_Station, Pixmap_Station, Masque_Station, FALSE, FALSE);
		gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Node_Station, (gpointer *) STATION);
		g_free (text[0]);
	}
}

GtkCTreeNode *Ajoute_Domaine_Serveur (int Machine)
{
	int       NumDomaine;
	int       trouver;
	gchar    *text[1];
	char      Tmp[20];
	GtkCTreeNode *Node_Parent;

	Node_Parent = NULL;
	/* la machine est un serveur et elle appartient à un domaine */
	if ((Liste_Machines[Machine].Serveur) && (Liste_Machines[Machine].Smb.DomainName[0] != 0)) {
		trouver = -1;
		NumDomaine = 0;
		while ((trouver == -1) && (NumDomaine < (MAX_DOMAINE - 1)) && (Liste_Domaines_Serveur[NumDomaine].ctree != NULL)) {
			if (strcmp (Liste_Domaines_Serveur[NumDomaine].DomainName, Liste_Machines[Machine].Smb.DomainName) == 0) {
				trouver = NumDomaine;
				printf ("[I] Le domaine Serveur existe deja %d\n", trouver);
			}
			else
				NumDomaine++;
		}
		Creer_Node_Serveur ();
		if (Liste_Domaines_Serveur[NumDomaine].ctree == NULL) {
			printf ("[I] Nouveau domaine station  [%s]\n", Liste_Machines[Machine].Smb.DomainName);
			sprintf (Tmp, "%s", Liste_Machines[Machine].Smb.DomainName);
			text[0] = g_strdup_printf (Tmp);
			Liste_Domaines_Serveur[NumDomaine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Serveur, NULL, text, 5, Pixmap_Domaine, Masque_Domaine, Pixmap_Domaine, Masque_Domaine, FALSE, FALSE);
			gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Liste_Domaines_Serveur[NumDomaine].ctree, (gpointer *) DOMAINE);
			g_free (text[0]);
			strcpy (Liste_Domaines_Serveur[NumDomaine].DomainName, Liste_Machines[Machine].Smb.DomainName);
			trouver = NumDomaine;
			Liste_Domaines_Serveur[NumDomaine].Nb_Machine = 1;
		}
		else {
			Liste_Domaines_Serveur[trouver].Nb_Machine++;

		}
		Node_Parent = Liste_Domaines_Serveur[trouver].ctree;
	}
	return Node_Parent;
}


GtkCTreeNode *Ajoute_Domaine_Station (int Machine)
{
	int       NumDomaine;
	int       trouver;
	gchar    *text[1];
	char      Tmp[20];
	GtkCTreeNode *Node_Parent;


	Node_Parent = NULL;
	/* la machine n'est pas un serveur et elle appartient à un domaine */
	if ((!Liste_Machines[Machine].Serveur) && (Liste_Machines[Machine].Smb.DomainName[0] != 0)) {
		trouver = -1;
		NumDomaine = 0;
		while ((trouver == -1) && (NumDomaine < (MAX_DOMAINE - 1)) && (Liste_Domaines_Station[NumDomaine].ctree != NULL)) {
			if (strcmp (Liste_Domaines_Station[NumDomaine].DomainName, Liste_Machines[Machine].Smb.DomainName) == 0) {
				trouver = NumDomaine;
				printf ("[I] Le domaine Station existe deja %d\n", trouver);
			}
			else
				NumDomaine++;
		}
		Creer_Node_Station ();
		if (Liste_Domaines_Station[NumDomaine].ctree == NULL) {
			printf ("[I] Nouveau domaine station  [%s]\n", Liste_Machines[Machine].Smb.DomainName);
			sprintf (Tmp, "%s", Liste_Machines[Machine].Smb.DomainName);
			text[0] = g_strdup_printf (Tmp);
			Liste_Domaines_Station[NumDomaine].ctree = gtk_ctree_insert_node (GTK_CTREE (ctree_Liste_Machine), Node_Station, NULL, text, 5, Pixmap_Domaine, Masque_Domaine, Pixmap_Domaine, Masque_Domaine, FALSE, FALSE);
			gtk_ctree_node_set_row_data (GTK_CTREE (ctree_Liste_Machine), Liste_Domaines_Station[NumDomaine].ctree, (gpointer *) DOMAINE);
			g_free (text[0]);
			strcpy (Liste_Domaines_Station[NumDomaine].DomainName, Liste_Machines[Machine].Smb.DomainName);
			trouver = NumDomaine;
			Liste_Domaines_Station[NumDomaine].Nb_Machine = 1;
		}
		else {
			Liste_Domaines_Station[trouver].Nb_Machine++;
		}
		Node_Parent = Liste_Domaines_Station[trouver].ctree;
	}
	return Node_Parent;
}
